#! /usr/bin/env python
#
# Copyright (c) 2013-2014 Wind River Systems, Inc.
#
# The right to copy, distribute, modify, or otherwise make use
# of this software may be licensed only pursuant to the terms
# of an applicable Wind River license agreement.
#


import argparse
import sys
import json
import time
import httplib2
import os.path
import tsconfig.tsconfig as tsc

HTTP_SUCCESS = 200

parser = argparse.ArgumentParser(description='Check AVS readiness.')
parser.add_argument('--force', action='store_true',
                    help='Run regardless of compute config status')
args, unknown_args = parser.parse_known_args()


class StatusError(Exception):
    """Base class for status errors"""
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


def decode(input):
    """
    Convert unicode strings to ascii byte strings.
    """
    if isinstance(input, dict):
        return {decode(key): decode(value) for key, value in input.iteritems()}
    elif isinstance(input, list):
        return [decode(element) for element in input]
    elif isinstance(input, unicode):
        return input.encode('utf-8')
    else:
        return input


# This small request code should be updated to use the vswitch python client
# API once it is available.

def main():
    url = "http://localhost:9000/v1/engine/stats"
    headers = {'Accept': 'application/json'}
    method = "GET"
    retry = 30
    status = 1

    # Check added for combo load where the controller is also a compute
    # but we don't want this script to run unless compute configuration
    # is complete.
    config_complete = os.path.exists(tsc.VOLATILE_WORKER_CONFIG_COMPLETE)
    if (not args.force) and config_complete is not True:
        sys.exit(0)

    while retry > 0:
        try:
            http = httplib2.Http()
            response, content = http.request(uri=url, method=method, headers=headers)
            if response.status != HTTP_SUCCESS:
                raise StatusError("failed to get engine stats: " + response.reason)

            engines = decode(json.loads(content))
            if not engines or len(engines) == 0:
                raise StatusError("no engines configured")

            for engine in engines:
                if (engine['cycles-busy'] + engine['cycles-idle']) == 0:
                    raise StatusError("engine {} is not running".format(engine['uuid']))

            # minimal checks seem ok, report a pass status and stop
            status = 0
            break

        except Exception as e:
            sys.stderr.write("failed to check vswitch status: {}\n".format(e))
            retry -= 1
            time.sleep(1)

    # exit with the status
    sys.exit(status)

if __name__ == '__main__':
    main()
